@@ -27,6 +27,7 @@
 
 #include <glib/gi18n.h>
 #include <gio/gio.h>
+#include <gdk/gdkx.h>
 
 #include <matemenu-tree.h>
 
@@ -1123,14 +1124,12 @@ panel_addto_selection_changed (GtkTreeSelection *selection,
 	gtk_widget_set_sensitive (GTK_WIDGET (dialog->add_button), TRUE);
 
 	if (data->type == PANEL_ADDTO_LAUNCHER_MENU) {
-		gtk_button_set_label (GTK_BUTTON (dialog->add_button),
-				      GTK_STOCK_GO_FORWARD);
+		gtk_button_set_image (GTK_BUTTON (dialog->add_button),
+				      gtk_image_new_from_icon_name ("go-next", GTK_ICON_SIZE_BUTTON));
 	} else {
-		gtk_button_set_label (GTK_BUTTON (dialog->add_button),
-				      GTK_STOCK_ADD);
+		gtk_button_set_image (GTK_BUTTON (dialog->add_button),
+				      gtk_image_new_from_icon_name ("list-add", GTK_ICON_SIZE_BUTTON));
 	}
-	gtk_button_set_use_stock (GTK_BUTTON (dialog->add_button),
-				  TRUE);
 
 	/* only allow dragging applets if we can add applets */
 	if (panel_profile_id_lists_are_writable ()) {
@@ -1222,15 +1221,15 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 
 	dialog->addto_dialog = gtk_dialog_new ();
 	gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
-			       GTK_STOCK_HELP, GTK_RESPONSE_HELP);
+			       "gtk-help", GTK_RESPONSE_HELP);
 	dialog->back_button = gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
-						     GTK_STOCK_GO_BACK,
+						     "gtk-go-back",
 						     PANEL_ADDTO_RESPONSE_BACK);
 	dialog->add_button = gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
-						     GTK_STOCK_ADD,
+						     "gtk-add",
 						     PANEL_ADDTO_RESPONSE_ADD);
 	gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
-			       GTK_STOCK_CLOSE,
+			       "gtk-close",
 			       GTK_RESPONSE_CLOSE);
 	gtk_widget_set_sensitive (GTK_WIDGET (dialog->add_button), FALSE);
 
@@ -1255,12 +1254,8 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 	gtk_box_pack_start (GTK_BOX (inner_vbox), find_hbox, FALSE, FALSE, 0);
 
 	dialog->label = gtk_label_new_with_mnemonic ("");
-#if GTK_CHECK_VERSION (3, 16, 0)
 	gtk_label_set_xalign (GTK_LABEL (dialog->label), 0.0);
 	gtk_label_set_yalign (GTK_LABEL (dialog->label), 0.5);
-#else
-	gtk_misc_set_alignment (GTK_MISC (dialog->label), 0.0, 0.5);
-#endif
 	gtk_label_set_use_markup (GTK_LABEL (dialog->label), TRUE);
 
 	gtk_box_pack_start (GTK_BOX (find_hbox), dialog->label,
@@ -1373,10 +1368,7 @@ panel_addto_present (GtkMenuItem *item,
 				     panel_addto_dialog_quark);
 
 	screen = gtk_window_get_screen (GTK_WINDOW (toplevel));
-
-	gdk_window_get_geometry (gdk_screen_get_root_window (screen), NULL, NULL,
-				 NULL, &screen_height);
-
+	screen_height = HeightOfScreen (gdk_x11_screen_get_xscreen (screen));
 	height = MIN (MAX_ADDTOPANEL_HEIGHT, 3 * (screen_height / 4));
 
 	if (!dialog) {
